import os
import re
from typing import Optional, Tuple

from .config import Config
from .git import Client as GitClient
from .aws import Client as AwsClient


class Repository:
    """
    Understands CodeCommit repositories
    """

    def __init__(self, path: Optional[str] = None) -> None:
        self.__remote: str = ""
        self.__aws_region: str = ""
        self.__aws_profile: str = ""
        self.__repository_name: str = ""
        self.__branch: str = ""

        if not path:
            path = os.getcwd()

        self.__git = GitClient(path)

    def __config(self, method: str) -> Optional[str]:
        return getattr(Config, method)(self.aws_profile)

    @property
    def supported(self) -> bool:
        return self.remote.startswith("codecommit::")

    @property
    def remote(self) -> str:
        if not self.__remote:
            self.__remote = self.__git.remote("origin")

            if self.supported:
                self.__extract_from_remote(self.__remote)

        return self.__remote

    def __extract_from_remote(self, remote: str) -> None:
        def resolve(resolver: Optional[re.Match], index: int) -> str:
            return resolver.group(index) if resolver else ""

        # This can use some more error handling
        match = re.search(r"^codecommit::(.*)://(.*)@(.*)$", remote)
        self.__aws_region = resolve(match, 1)
        self.__aws_profile = resolve(match, 2)
        self.__repository_name = resolve(match, 3)

    @property
    def aws_region(self) -> str:
        return self.__aws_region

    @property
    def aws_profile(self) -> str:
        return self.__aws_profile

    @property
    def repository_name(self) -> str:
        return self.__repository_name

    @property
    def branch(self) -> str:
        if not self.__branch:
            self.__branch = self.__git.current_branch

        return self.__branch

    @property
    def destination(self) -> str:
        destination = self.__config("destination_branch")
        return destination if destination else ""

    def pull_request_information(self) -> Tuple[str, str]:
        title_postfix = ""
        commits = self.__git.get_commit_messages(destination_branch=self.destination)

        issues = ", ".join(commits.issues)
        description = list(map(lambda commit: commit.message.subject, commits))
        if issues:
            description.append(f"\nIssues: {issues}")
            title_postfix = f" ({issues})"

        title = f"{commits.first.message.subject}{title_postfix}"
        return title, "\n".join(description)

    def create_pull_request(self, title: str, description: str) -> str:
        client = AwsClient(profile=self.__aws_profile, region=self.__aws_region)
        response = client.create_pull_request(
            title=title,
            description=description,
            repository=self.repository_name,
            source=self.branch,
            destination=self.destination,
        )
        return f"https://{self.__aws_region}.console.aws.amazon.com/codesuite/codecommit/repositories/{self.repository_name}/pull-requests/{response.get('pullRequestId')}/details?region={self.__aws_region}"



from typing import Optional

import click

from pull_request_codecommit.repository import Repository


@click.command()
@click.option("-r", "--repository-path", default=None)
def main(repository_path: Optional[str]) -> None:
    """
    pull-request-codecommit
    """
    repo = Repository(repository_path)

    if not repo.supported:
        raise click.ClickException("The repository is not compatible with this tool!")

    click.echo(f"Remote: {repo.remote}")
    click.echo(f"Region: {repo.aws_region}")
    click.echo(f"Profile: {repo.aws_profile}")
    click.echo(f"Repo: {repo.repository_name}")
    click.echo(f"Branch: {repo.branch}")
    click.echo()
    title, description = repo.pull_request_information()

    message = click.edit(f"{title}\n\n{description}")

    if message is None:
        raise click.ClickException("Pull request was not created")

    title = message.splitlines()[0]
    description = "\n".join(message.splitlines()[1:])
    link = repo.create_pull_request(title, description)
    click.echo(f"Please review/approve: {link}")
    click.echo()
    click.echo(click.style(title, bold=True))
    click.echo(description)


if __name__ == "__main__":
    main()



import re


class Message:
    """
    Understands git message that follow the conventional commits standard.

    See: https://www.conventionalcommits.org/en/v1.0.0/
    """

    def __init__(self, message: str) -> None:
        lines = []

        for line in iter(message.splitlines()):
            lines.append(line.strip(" "))

        self.__lines = list(filter(None, lines))
        self.__message = "\n".join(self.__lines)

    @property
    def issue(self) -> str:
        """
        Issue reference should be in the footer of the commit message
        """
        match = re.search(r"(?i)issue(:|) (.*)", self.__lines[-1])
        return f"{match.group(2)}" if match else ""

    @property
    def subject(self) -> str:
        """
        Subject is the first line of the commit message
        """
        return self.__lines[0]



import json
from io import TextIOWrapper, BytesIO
from unittest.mock import patch, MagicMock

import pytest
from click.testing import CliRunner
from pull_request_codecommit import main
from pull_request_codecommit.config import configparser
from pull_request_codecommit.git import Commits
from .test_git_client import COMMITS, COMMITS_NO_ISSUES


def edit_message(message: str) -> str:
    return message


@pytest.mark.parametrize(
    "region, profile, config, commits",
    [
        (
            "eu-west-1",
            "my-profile",
            b"[default]\nbranch: my-main\n[profile my-profile]\nbranch: my-master",
            COMMITS,
        ),
        (
            "eu-central-1",
            "my-profile",
            b"[default]\nbranch: my-main\n[profile my-profile]\nbranch: my-master",
            COMMITS_NO_ISSUES,
        ),
        (
            "eu-west-1",
            "my-other-profile",
            b"[default]\nbranch: my-main\n[profile my-profile]\nbranch: my-master",
            COMMITS_NO_ISSUES,
        ),
        (
            "eu-central-1",
            "my-other-profile",
            b"[default]\nbranch: my-main\n[profile my-profile]\nbranch: my-master",
            COMMITS,
        ),
    ],
)
@patch("pull_request_codecommit.aws.client.subprocess.run")
@patch("pull_request_codecommit.repository.GitClient")
@patch("pull_request_codecommit.click.edit")
def test_invoke(
    mock_edit: MagicMock,
    mock_git_client: MagicMock,
    mock_aws_client: MagicMock,
    region: str,
    profile: str,
    config: bytes,
    commits: str,
) -> None:
    mock_edit.side_effect = edit_message
    mock_git_client.return_value.get_commit_messages.return_value = Commits(commits)

    mock_git_client.return_value.remote.return_value = (
        f"codecommit::{region}://{profile}@my-repository"
    )
    mock_git_client.return_value.current_branch.return_value = "feat/my-feature"
    configparser.open = MagicMock(return_value=TextIOWrapper(BytesIO(config)))  # type: ignore

    def execute(parameters, stdout):
        assert -1 == stdout
        mock_stdout = MagicMock()
        data = {"pullRequest": {"pullRequestId": 1}}
        mock_stdout.stdout = bytes(json.dumps(data), "utf-8")
        return mock_stdout

    mock_aws_client.side_effect = execute

    runner = CliRunner()
    result = runner.invoke(main, [])
    assert result.exit_code == 0


@pytest.mark.parametrize(
    "region, profile, config",
    [
        (
            "eu-west-1",
            "my-profile",
            b"[default]\nbranch: my-main\n[profile my-profile]\nbranch: my-master",
        ),
        (
            "eu-central-1",
            "my-profile",
            b"[default]\nbranch: my-main\n[profile my-profile]\nbranch: my-master",
        ),
        (
            "eu-west-1",
            "my-other-profile",
            b"[default]\nbranch: my-main\n[profile my-profile]\nbranch: my-master",
        ),
        (
            "eu-central-1",
            "my-other-profile",
            b"[default]\nbranch: my-main\n[profile my-profile]\nbranch: my-master",
        ),
    ],
)
@patch("pull_request_codecommit.aws.client.subprocess.run")
@patch("pull_request_codecommit.repository.GitClient")
@patch("pull_request_codecommit.click.edit")
def test_invoke_with_path(
    mock_edit: MagicMock,
    mock_git_client: MagicMock,
    mock_aws_client: MagicMock,
    region: str,
    profile: str,
    config: bytes,
) -> None:
    mock_edit.side_effect = edit_message
    mock_git_client.return_value.remote.return_value = (
        f"codecommit::{region}://{profile}@my-repository"
    )
    mock_git_client.return_value.current_branch.return_value = "feat/my-feature"
    configparser.open = MagicMock(return_value=TextIOWrapper(BytesIO(config)))  # type: ignore

    def execute(parameters, stdout):
        assert -1 == stdout
        mock_stdout = MagicMock()
        data = {"pullRequest": {"pullRequestId": 1}}
        mock_stdout.stdout = bytes(json.dumps(data), "utf-8")
        return mock_stdout

    mock_aws_client.side_effect = execute

    runner = CliRunner()
    result = runner.invoke(main, ["--repository-path", "./some/path/to/repo"])
    assert result.exit_code == 0


@patch("pull_request_codecommit.repository.GitClient")
def test_invoke_github(mock_git_client: MagicMock) -> None:
    mock_git_client.return_value.remote.return_value = (
        "git@github.com:Nr18/pull-request-codecommit.git"
    )
    mock_git_client.return_value.current_branch.return_value = "feat/my-feature"

    runner = CliRunner()
    result = runner.invoke(main, [])
    assert result.exit_code == 1


@patch("pull_request_codecommit.repository.GitClient")
@patch("pull_request_codecommit.click.edit")
def test_invoke_quit_edit(
    mock_edit: MagicMock,
    mock_git_client: MagicMock,
) -> None:
    mock_edit.return_value = None
    mock_git_client.return_value.remote.return_value = (
        f"codecommit::eu-west-1://my-profile@my-repository"
    )
    mock_git_client.return_value.current_branch.return_value = "feat/my-feature"
    config = b"[default]\nbranch: my-main\n[profile my-profile]\nbranch: my-master"
    configparser.open = MagicMock(return_value=TextIOWrapper(BytesIO(config)))  # type: ignore

    runner = CliRunner()
    result = runner.invoke(main)
    assert result.exit_code == 1



import pytest
from unittest.mock import patch, MagicMock

from pull_request_codecommit.git import Commit
from pull_request_codecommit.git.client import Client

COMMIT2 = """commit my-hash-1
Author: John Doe <john@doe.nl>
Date:   Fri Jan 21 21:01:00 2022 +0100

    feat: my first commit"""

COMMIT1 = """commit my-hash-2
Author: John Doe <john@doe.nl>
Date:   Fri Jan 21 21:01:00 2022 +0100

    feat: my second commit"""

COMMITS = f"{COMMIT1}\n\n    Issue #1\n\n{COMMIT2}\n\n    Issue #2"
COMMITS_NO_ISSUES = f"{COMMIT1}\n\n{COMMIT2}"


def test_git_client() -> None:
    with pytest.raises(Exception) as exc:
        Client("fake-path")

    assert str(exc.value) == "The fake-path is not a directory."


@pytest.mark.parametrize(
    "path, remote",
    [
        ("my-path", "codecommit::eu-west-1://my-profile@my-repository"),
        (
            "my-other-path",
            "codecommit::eu-central-1://my-other-profile@my-other-repository",
        ),
    ],
)
@patch("pull_request_codecommit.git.client.subprocess.run")
@patch("pull_request_codecommit.git.client.os.path.isdir")
def test_remote(
    mock_isdir: MagicMock, mock_run: MagicMock, path: str, remote: str
) -> None:
    mock_isdir.return_value = True

    def execute(parameters, cwd, stdout):
        assert path == cwd
        assert -1 == stdout
        mock_stdout = MagicMock()
        mock_stdout.stdout = bytes(f"\n\n{remote}\n\n", "utf-8")
        return mock_stdout

    mock_run.side_effect = execute

    client = Client(path)

    assert client.remote() == remote
    assert client.remote("origin") == remote


@pytest.mark.parametrize(
    "path, branch",
    [
        ("my-path", "main"),
        (
            "my-other-path",
            "master",
        ),
    ],
)
@patch("pull_request_codecommit.git.client.subprocess.run")
@patch("pull_request_codecommit.git.client.os.path.isdir")
def test_current_branch(
    mock_isdir: MagicMock, mock_run: MagicMock, path: str, branch: str
) -> None:
    mock_isdir.return_value = True

    def execute(parameters, cwd, stdout):
        assert path == cwd
        assert -1 == stdout
        mock_stdout = MagicMock()
        mock_stdout.stdout = bytes(f"\n\n{branch}\n\n", "utf-8")
        return mock_stdout

    mock_run.side_effect = execute
    client = Client(path)
    assert client.current_branch == branch


@pytest.mark.parametrize(
    "path, branch",
    [
        ("my-path", "main"),
        (
            "my-other-path",
            "master",
        ),
    ],
)
@patch("pull_request_codecommit.git.client.subprocess.run")
@patch("pull_request_codecommit.git.client.os.path.isdir")
def test_get_commit_messages(
    mock_isdir: MagicMock, mock_run: MagicMock, path: str, branch: str
) -> None:
    mock_isdir.return_value = True

    def execute(parameters, cwd, stdout):
        assert path == cwd
        assert -1 == stdout
        mock_stdout = MagicMock()
        mock_stdout.stdout = bytes(COMMITS, "utf-8")
        return mock_stdout

    mock_run.side_effect = execute
    client = Client(path)

    commits = client.get_commit_messages("main")
    first: Commit = next(commits)
    second: Commit = next(commits)

    assert first.message.subject == "feat: my first commit"
    assert second.message.subject == "feat: my second commit"



from __future__ import annotations

import json
from typing import List
import subprocess


class Client:
    """
    Understands aws operations

    We use the AWS CLI for these operations so that we can use tha MFA toke cache from the cli.
    """

    def __init__(self, profile: str, region: str) -> None:
        self.__profile = profile
        self.__region = region

    def __execute(self, parameters: List[str]) -> str:
        command = ["aws", "--profile", self.__profile, "--region", self.__region]
        command.extend(parameters)
        response = subprocess.run(command, stdout=subprocess.PIPE)

        return response.stdout.decode("utf-8").strip("\n")

    def create_pull_request(
        self,
        title: str,
        description: str,
        repository: str,
        source: str,
        destination: str,
    ) -> dict:
        command = [
            "codecommit",
            "create-pull-request",
            "--title",
            title,
            "--description",
            description,
            "--targets",
            f"repositoryName={repository}, sourceReference={source}, destinationReference={destination}",
        ]
        response = self.__execute(command)
        data = json.loads(response)
        return data.get("pullRequest")
